"""OpenAI ChatCompletions models categorized by audio support.

This file is auto-generated by scripts/update_openai_completions_model_features.py
Run that script to update this map when OpenAI releases new models.
"""

from typing import Literal

AudioFeatureSupport = Literal["supports_audio", "no_audio_support", "unavailable"]

MODEL_FEATURES: dict[str, AudioFeatureSupport] = {
    "chatgpt-4o-latest": "no_audio_support",
    "codex-mini-latest": "unavailable",
    "gpt-3.5-turbo": "no_audio_support",
    "gpt-3.5-turbo-0125": "no_audio_support",
    "gpt-3.5-turbo-0301": "unavailable",
    "gpt-3.5-turbo-0613": "unavailable",
    "gpt-3.5-turbo-1106": "no_audio_support",
    "gpt-3.5-turbo-16k": "no_audio_support",
    "gpt-3.5-turbo-16k-0613": "unavailable",
    "gpt-4": "no_audio_support",
    "gpt-4-0125-preview": "no_audio_support",
    "gpt-4-0314": "unavailable",
    "gpt-4-0613": "no_audio_support",
    "gpt-4-1106-preview": "no_audio_support",
    "gpt-4-32k": "unavailable",
    "gpt-4-32k-0314": "unavailable",
    "gpt-4-32k-0613": "unavailable",
    "gpt-4-turbo": "no_audio_support",
    "gpt-4-turbo-2024-04-09": "no_audio_support",
    "gpt-4-turbo-preview": "no_audio_support",
    "gpt-4-vision-preview": "unavailable",
    "gpt-4.1": "no_audio_support",
    "gpt-4.1-2025-04-14": "no_audio_support",
    "gpt-4.1-mini": "no_audio_support",
    "gpt-4.1-mini-2025-04-14": "no_audio_support",
    "gpt-4.1-nano": "no_audio_support",
    "gpt-4.1-nano-2025-04-14": "no_audio_support",
    "gpt-4o": "no_audio_support",
    "gpt-4o-2024-05-13": "no_audio_support",
    "gpt-4o-2024-08-06": "no_audio_support",
    "gpt-4o-2024-11-20": "no_audio_support",
    "gpt-4o-audio-preview": "supports_audio",
    "gpt-4o-audio-preview-2024-10-01": "supports_audio",
    "gpt-4o-audio-preview-2024-12-17": "supports_audio",
    "gpt-4o-audio-preview-2025-06-03": "supports_audio",
    "gpt-4o-mini": "no_audio_support",
    "gpt-4o-mini-2024-07-18": "no_audio_support",
    "gpt-4o-mini-audio-preview": "supports_audio",
    "gpt-4o-mini-audio-preview-2024-12-17": "supports_audio",
    "gpt-4o-mini-search-preview": "no_audio_support",
    "gpt-4o-mini-search-preview-2025-03-11": "no_audio_support",
    "gpt-4o-search-preview": "no_audio_support",
    "gpt-4o-search-preview-2025-03-11": "no_audio_support",
    "gpt-5": "no_audio_support",
    "gpt-5-2025-08-07": "no_audio_support",
    "gpt-5-chat-latest": "no_audio_support",
    "gpt-5-mini": "no_audio_support",
    "gpt-5-mini-2025-08-07": "no_audio_support",
    "gpt-5-nano": "no_audio_support",
    "gpt-5-nano-2025-08-07": "no_audio_support",
    "o1": "no_audio_support",
    "o1-2024-12-17": "no_audio_support",
    "o1-mini": "no_audio_support",
    "o1-mini-2024-09-12": "no_audio_support",
    "o1-preview": "unavailable",
    "o1-preview-2024-09-12": "unavailable",
    "o3": "no_audio_support",
    "o3-2025-04-16": "no_audio_support",
    "o3-mini": "no_audio_support",
    "o3-mini-2025-01-31": "no_audio_support",
    "o4-mini": "no_audio_support",
    "o4-mini-2025-04-16": "no_audio_support",
}
"""Map of model IDs to their status.

- "supports_audio": model exists and supports audio inputs
- "no_audio_support": model exists but does not support audio inputs
- "unavailable": model has been deprecated or removed
- If model not in map: assume it exists and supports audio (optimistic default)
"""
